package ru.dbotthepony.mc.otm.datagen.models

import net.minecraft.world.item.DyeColor
import net.minecraft.world.level.block.Block
import net.minecraftforge.client.model.generators.BlockModelProvider
import net.minecraftforge.data.event.GatherDataEvent
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.modLocation
import java.util.LinkedList

class MatteryBlockModelProvider(event: GatherDataEvent) : BlockModelProvider(event.generator.packOutput, DataGen.MOD_ID, event.existingFileHelper) {
	private val callbacks = LinkedList<(MatteryBlockModelProvider) -> Unit>()

	fun exec(callback: (MatteryBlockModelProvider) -> Unit) {
		callbacks.add(callback)
	}

	override fun registerModels() {
		for (callback in callbacks) {
			callback(this)
		}
	}

	override fun getName(): String {
		return "Block Models: $modid"
	}

	fun decorativeGlassAll(blocks: Collection<Block>) {
		for (block in blocks) {
			exec {
				cubeAll(block.registryName!!.path, modLocation("block/decorative/${block.registryName!!.path}")).renderType("translucent")
			}
		}
	}

	fun decorativeGlassAll(vararg blocks: Block) {
		for (block in blocks) {
			exec {
				cubeAll(block.registryName!!.path, modLocation("block/decorative/${block.registryName!!.path}")).renderType("translucent")
			}
		}
	}

	fun decorativeCubeAll(vararg blocks: Block) {
		for (block in blocks) {
			exec {
				cubeAll(block.registryName!!.path, modLocation("block/decorative/${block.registryName!!.path}"))
			}
		}
	}

	fun decorativeCubeAll(subdir: String, vararg blocks: Block) {
		for (block in blocks) {
			exec {
				cubeAll(block.registryName!!.path, modLocation("block/decorative/${subdir}/${block.registryName!!.path}"))
			}
		}
	}

	fun decorativeCubeAll(subdir: String, suffix: String, vararg blocks: Block) {
		for (block in blocks) {
			exec {
				cubeAll(block.registryName!!.path, modLocation("block/decorative/${subdir}/${block.registryName!!.path}$suffix"))
			}
		}
	}

	fun decorativeCubeAll(blocks: Collection<Block>) {
		for (block in blocks) {
			exec {
				cubeAll(block.registryName!!.path, modLocation("block/decorative/${block.registryName!!.path}"))
			}
		}
	}

	fun decorativeCubeAll(block: Block, texture: String) {
		exec {
			cubeAll(block.registryName!!.path, modLocation("block/decorative/$texture"))
		}
	}

	fun column(block: Block, end: String, side: String) {
		exec {
			cubeColumn(block.registryName!!.path, modLocation(end), modLocation(side))
		}
	}

	fun decorativeColumn(it: Block, end: String, side: String) {
		column(it, "block/decorative/$end", "block/decorative/$side")
	}

	fun resourceCubeAll(vararg blocks: Block) {
		for (block in blocks) {
			exec {
				cubeAll(block.registryName!!.path, modLocation("block/resource/${block.registryName!!.path}"))
			}
		}
	}

	fun coloredMachine(blocks: Map<DyeColor?, Block>, textureKeys: Collection<String>) {
		val base = blocks[null]!!.registryName!!

		for ((color, block) in blocks.entries) {
			if (color == null) continue

			exec {
				val model = withExistingParent(block.registryName!!.path, base)

				for (key in textureKeys) {
					model.texture(key, modLocation("block/${base.path}/${color.name.lowercase()}"))
				}
			}
		}
	}

	fun colored(modelName: String, textureKeys: Map<String, String>) {
		for (color in DyeColor.entries) {
			exec {
				val model = withExistingParent(modelName + "_${color.name.lowercase()}", modLocation(modelName))

				for ((key, value) in textureKeys) {
					model.texture(key, modLocation("block/$value/${color.name.lowercase()}"))
				}
			}
		}
	}

	fun coloredMachineCombined(modelName: String, textureName: String, states: Collection<String>, textureKeys: Collection<String>) {
		exec {
			for (color in DyeColor.entries) {
				for (state in states) {
					val model = withExistingParent(modelName + "_${color.name.lowercase()}_$state", modLocation(modelName + "_$state"))

					for (key in textureKeys) {
						model.texture(key, modLocation("block/$textureName/${color.name.lowercase()}"))
					}
				}
			}
		}
	}
}
